<?php

declare(strict_types=1);

namespace Drupal\config_change_track\EventSubscriber;

use Drupal\config_change_track\Tracker;
use Drupal\Core\Config\ConfigCrudEvent;
use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\ConfigImporterEvent;
use Drupal\Core\Config\ConfigRenameEvent;
use Drupal\Core\DependencyInjection\ClassResolverInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Detects changes in config.
 *
 * @todo This could check on save if there's actually a difference.
 */
final class ConfigChangeSubscriber implements EventSubscriberInterface {

  public function __construct(
    private readonly ClassResolverInterface $classResolver,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      ConfigEvents::SAVE => ['onCrud'],
      ConfigEvents::DELETE => ['onCrud'],
      ConfigEvents::RENAME => ['onRename'],
      ConfigEvents::IMPORT => ['onImport'],
    ];
  }

  /**
   * Responds to a config CRUD event.
   *
   * @param \Drupal\Core\Config\ConfigCrudEvent $event
   *   The event.
   */
  public function onCrud(ConfigCrudEvent $event): void {
    $this->getTracker()->setLatestUpdate();
  }

  /**
   * Responds to config being renamed.
   *
   * @param \Drupal\Core\Config\ConfigRenameEvent $event
   *   The event.
   */
  public function onRename(ConfigRenameEvent $event): void {
    $this->getTracker()->setLatestUpdate();
  }

  /**
   * Responds to config being imported.
   *
   * @param \Drupal\Core\Config\ConfigImporterEvent $event
   *   The event.
   */
  public function onImport(ConfigImporterEvent $event): void {
    $this->getTracker()->setLatestUpdate();
  }

  /**
   * Returns the tracker.
   */
  private function getTracker(): Tracker {
    return $this->classResolver->getInstanceFromDefinition(Tracker::class);
  }

}
